@@ -1,4 +1,7 @@
 package Giddy;
+BEGIN {
+  $Giddy::VERSION = '0.01';
+}
 
 # ABSTRACT: Schema-less, versioned media/document database based on Git.
 
@@ -6,19 +9,16 @@ use Any::Moose;
 use namespace::autoclean;
 
 use Carp;
-use Git::Repository 1.18;
+use File::Spec;
 use Giddy::Database;
 
-our $VERSION = "0.013_001";
-$VERSION = eval $VERSION;
-
 =head1 NAME
 
 Giddy - Schema-less, versioned media/document database based on Git.
 
 =head1 VERSION
 
-version 0.013_001
+version 0.01
 
 =head1 SYNOPSIS
 
@@ -31,13 +31,12 @@ version 0.013_001
 =head1 DESCRIPTION
 
 WARNING: THIS IS ALPHA SOFTWARE, RELEASED FOR TESTING PURPOSES ONLY. DO
-NOT USE IT ON A PRODUCTION ENVIRONMENT YET. IT'S INCOMPLETE, BUG-RIDDEN, LIKELY
-TO CHANGE, AND WILL RUN OVER YOUR CAT.
+NOT USE IT ON A PRODUCTION ENVIRONMENT YET. IT'S INCOMPLETE, BUG-RIDDEN,
+AND WILL RUN OVER YOUR CAT.
 
-Giddy is a schema-less (as in NoSQL), versioned database system for Unix-like
-operating systems, built on top of Git. A database in Giddy is simply a Git
-repository, providing the database with automatic, comprehensive versioning and
-distributive capabilities.
+Giddy is a schema-less (as in NoSQL), versioned database system built on
+top of Git. A database in Giddy is simply a Git repository, providing the
+database with automatic, comprehensive versioning and distributive capabilities.
 
 As opposed to most modern database systems, Giddy aims to be human editable.
 One can create/edit/delete database entries with nothing but a text editor
@@ -83,9 +82,7 @@ Returns a L<Giddy::Database> object tied to a Git repository located on the file
 system. Please provide full path names to prevent potential problems.
 
 If the path doesn't exist, Giddy will attempt to create it and initialize it
-as a Git repository. It will also create an empty file called ".giddy" inside
-the database and perform an initial commit. You can safely remove that file
-after that.
+as a Git repository.
 
 =cut
 
@@ -99,21 +96,12 @@ sub get_database {
 		unless $path;
 
 	# is this an existing database or a new one?
-	if (-d $path && -d "${path}/.git") {
+	if (-d $path && -d File::Spec->catdir($path, '.git')) {
 		# existing
 		return Giddy::Database->new(_repo => Git::Repository->new(work_tree => $path));
 	} else {
-		# new one, let's create the repository's directory
-		Git::Repository->run('init', $path)
-			|| croak "Failed creating new Giddy repository, please check provided path is valid and that you have appropriate permissions to create the repository.";
-		my $db = Giddy::Database->new(_repo => Git::Repository->new(work_tree => $path));
-		
-		# create an empty .giddy file, stage it and commit, so our database will be "live"
-		$db->_touch('.giddy');
-		$db->stage('.giddy');
-		$db->commit('initial commit');
-
-		return $db;
+		# new one
+		return Giddy::Database->new(_repo => Git::Repository->create(init => $path));
 	}
 }
 
