@@ -1,4 +1,7 @@
 package Giddy::Role::DocumentLoader;
+BEGIN {
+  $Giddy::Role::DocumentLoader::VERSION = '0.01';
+}
 
 use Any::Moose 'Role';
 use namespace::autoclean;
@@ -8,11 +11,7 @@ use Encode;
 use Try::Tiny;
 use YAML::XS;
 
-our $VERSION = "0.013_001";
-$VERSION = eval $VERSION;
-
-requires 'db';
-requires '_path_to';
+requires '_database';
 
 =head1 NAME
 
@@ -20,7 +19,7 @@ Giddy::Role::DocumentLoader - Provides document loading methods for Giddy::Colle
 
 =head1 VERSION
 
-version 0.013_001
+version 0.01
 
 =head1 SYNOPSIS
 
@@ -30,21 +29,24 @@ version 0.013_001
 
 This role provides document loading capabilities to L<Giddy::Collection> and L<Giddy::Collection::InMemory>.
 
-Requires the attributes 'db' and '_path_to' to be implemented by consuming classes.
-
 =head1 METHODS
 
-=head3 _load_document_file( $name )
+=head3 _load_document_file( $path )
 
 =cut
 
 sub _load_document_file {
-	my ($self, $name) = @_;
+	my ($self, $path) = @_;
 
-	my $content = ''.$self->db->_repo->run('show', 'HEAD:'.$self->_path_to($name));
+	my $spath = $path;
+	$spath =~ s!^/!!;
+
+	my $content = ''.$self->_database->_repo->run('show', 'HEAD:'.$spath);
+
+	return unless $content;
 
 	my ($yaml, $body) = ('', '');
-	if ($content && $content =~ m/\n\n/) {
+	if ($content =~ m/\n\n/) {
 		($yaml, $body) = ($`, $');
 	} else {
 		$body = $content;
@@ -53,44 +55,38 @@ sub _load_document_file {
 	return try {
 		my $doc = Load("---\n$yaml");
 		$doc->{_body} = $body;
-		$doc->{_name} = $name;
-		$doc->{_coll} = $self->path;
+		$doc->{_name} = ($path =~ m!/([^/]+)$!)[0];
 		return $doc;
 	} catch {
-		return { _body => $body, _name => $name, _coll => $self->path };
+		return { _body => $body, _name => ($path =~ m!/([^/]+)$!)[0] };
 	};
 }
 
-=head3 _load_document_dir( $name, [ $skip_binary ] )
+=head3 _load_document_dir( $path, [ $skip_binary ] )
 
 =cut
 
 sub _load_document_dir {
-	my ($self, $name, $skip_bin) = @_;
+	my ($self, $path, $skip_bin) = @_;
+
+	my $spath = $path;
+	$spath =~ s!^/!!;
 
 	my $doc;
 
 	# try to load the attributes
-	my $yaml = $self->db->_repo->run('show', 'HEAD:'.$self->_path_to($name, 'attributes.yaml'));
-	croak "Can't find/read attributes.yaml file of document $name." unless $yaml;
+	my $yaml = $self->_database->_repo->run('show', 'HEAD:'.File::Spec->catfile($spath, 'attributes.yaml'));
+	croak "Can't find/read attributes.yaml file of document $path." unless $yaml;
 	$doc = try { Load($yaml) } catch { {} };
 
-	# load child collections
-	$doc->{_has_many} = [sort grep { !$self->db->_is_document_dir($self->_path_to($name, $_)) && !$self->db->_is_static_dir($self->_path_to($name, $_)) } $self->db->_list_dirs($self->_path_to($name))];
-
-	# load child documents
-	$doc->{_has_one} = [sort grep { $self->db->_is_document_dir($self->_path_to($name, $_)) } $self->db->_list_dirs($self->_path_to($name))];
-
 	# try to load binary files (unless we're skipping binary)
 	unless ($skip_bin) {
-		foreach (grep { $_ ne 'attributes.yaml' } $self->db->_list_files($self->_path_to($name))) {
-			$doc->{$_} = $self->_path_to($name, $_);
+		foreach (grep {!/^attributes\.yaml$/} $self->_database->_repo->run('ls-tree', '--name-only', "HEAD:".$spath)) {
+			$doc->{$_} = File::Spec->catfile($path, $_);
 		}
 	}
 
-	$doc->{_name} = $name
-		if $doc && scalar keys %$doc;
-	$doc->{_coll} = $self->path;
+	$doc->{_name} = ($path =~ m!/([^/]+)$!)[0] if $doc && scalar keys %$doc;
 
 	return $doc;
 }
