@@ -1,4 +1,7 @@
 package Giddy::Role::DocumentMatcher;
+BEGIN {
+  $Giddy::Role::DocumentMatcher::VERSION = '0.01';
+}
 
 use Any::Moose 'Role';
 use namespace::autoclean;
@@ -6,22 +9,16 @@ use namespace::autoclean;
 use Carp;
 use Data::Compare;
 use DateTime::Format::W3CDTF;
+use Path::Abstract qw/--no_0_093_warning/;
 use Try::Tiny;
 
-our $VERSION = "0.013_001";
-$VERSION = eval $VERSION;
-
-requires 'db';
-requires '_documents';
-requires '_load_document';
-
 =head1 NAME
 
 Giddy::Role::DocumentMatcher - Provides query parsing and document matching for Giddy::Collection
 
 =head1 VERSION
 
-version 0.013_001
+version 0.01
 
 =head1 SYNOPSIS
 
@@ -31,8 +28,6 @@ version 0.013_001
 
 This role provides document matching capabilities to L<Giddy::Collection> and L<Giddy::Collection::InMemory>.
 
-Requires the 'db', '_documents' and '_load_document' attributes/methods to be implemented by consuming classes.
-
 =head1 METHODS
 
 =head2 _match_by_name( $name, \%options )
@@ -49,41 +44,16 @@ sub _match_by_name {
 	$name eq '' && return $self->_documents;
 
 	my $docs = Tie::IxHash->new;
-
-	# if query is for equality then only one document can be matched
-	# and since we want this to be fast, let's search for it a little
-	# differently
-	if (!ref $name || (ref $name eq 'HASH' && scalar keys %$name == 1 && $name->{'$eq'})) {
-		# if the collection is in-memory, then we already have a
-		# hash-ref of all documents in it and we can just find out
-		# if it has the document we're searching for
-		my $n = ref $name ? $name->{'$eq'} : $name;
-		if (ref $self eq 'Giddy::Collection::InMemory') {
-			my $doc = $self->_documents->FETCH($n);
-			$docs->STORE($n => $doc)
-				if $doc;
-		} else {
-			# we're gonna use Git's cat-file command and see
-			# if we have such a document. Since this command
-			# croaks when the document doesn't exist, we're
-			# gonna eval it
-			try {
-				my $type = $self->db->_repo->run('cat-file', '-t', 'HEAD:'.$self->_path_to($n));
-				if ($type eq 'blob') {
-					$docs->STORE($n => 'file');
-				} elsif ($type eq 'tree') {
-					$docs->STORE($n => 'dir');
-				}
-			};
-		}
-	} else {
-		foreach (grep { $self->_attribute_matches({ _name => $_ }, '_name', $name) } $self->_documents->Keys) {
-			$docs->STORE($_ => $self->_documents->FETCH($_));
-		}
-
-		$docs->SortByKey;
+	foreach ($self->_documents->Keys) {
+		my $t = $self->_documents->FETCH($_);
+		my $doc_path = Path::Abstract->new($_);
+		my $doc_name = $doc_path->last;
+		$docs->STORE($_ => $t)
+			if $self->_attribute_matches({ _name => $doc_name }, '_name', $name);
 	}
 
+	$docs->SortByKey;
+
 	return $docs;
 }
 
@@ -98,7 +68,7 @@ sub _match_by_query {
 	$opts ||= {};
 
 	# return all documents if we don't really have a query
-	scalar keys %$query == 0 && return $self->_documents;
+	scalar keys %$query == 0 && $self->_documents;
 
 	my $docs = Tie::IxHash->new;
 	my $loaded = {};
