@@ -1,16 +1,19 @@
 package Giddy::Role::DocumentStorer;
+BEGIN {
+  $Giddy::Role::DocumentStorer::VERSION = '0.01';
+}
 
 use Any::Moose 'Role';
 use namespace::autoclean;
 
 use Carp;
+use Fcntl qw/:flock/;
+use File::Path qw/make_path/;
 use YAML::XS;
 
-our $VERSION = "0.013_001";
-$VERSION = eval $VERSION;
-
-requires 'db';
-requires '_path_to';
+requires 'path';
+requires '_database';
+requires '_spath';
 
 =head1 NAME
 
@@ -18,7 +21,7 @@ Giddy::Role::DocumentStorer - Provides document storing for Giddy::Collection
 
 =head1 VERSION
 
-version 0.013_001
+version 0.01
 
 =head1 SYNOPSIS
 
@@ -28,8 +31,6 @@ version 0.013_001
 
 This role provides document storing capabilities to L<Giddy::Collection> and L<Giddy::Collection::InMemory>.
 
-Requires the attributes 'db' and '_path_to' to be implemented by consuming classes.
-
 =head1 METHODS
 
 =head2 _store_document( $filename, \%attributes )
@@ -42,8 +43,9 @@ sub _store_document {
 	# don't allow the _name attribute
 	delete $attrs->{_name};
 
-	my $fpath = $self->_path_to($filename);
 	if (exists $attrs->{_body}) {
+		my $fpath = File::Spec->catfile($self->_database->_repo->work_tree, $self->_spath, $filename);
+
 		my $body = delete $attrs->{_body};
 
 		my $content = '';
@@ -53,19 +55,43 @@ sub _store_document {
 		$content =~ s/^---\n//;
 
 		# create the document
-		$self->db->_create_file($fpath, $content, 0664);
+		$self->_write_file($fpath, $content, 0664);
+
+		# mark the document for staging
+		$self->_database->mark(File::Spec->catfile($self->path, $filename));
 	} else {
+		my $fpath = File::Spec->catdir($self->_database->_repo->work_tree, $self->_spath, $filename);
+
 		# create the document directory
-		$self->db->_create_dir($fpath);
+		make_path($fpath, { mode => 0755 });
 
 		# create the attributes file
 		my $yaml = Dump($attrs);
 		$yaml =~ s/^---\n//;
-		$self->db->_create_file($fpath.'/attributes.yaml', $yaml, 0664);
-	}
+		$self->_write_file(File::Spec->catfile($fpath, 'attributes.yaml'), $yaml, 0664);
 
-	# stage the document
-	$self->db->stage($fpath);
+		# mark the document for staging
+		$self->_database->mark(File::Spec->catdir($self->path, $filename));
+	}
+}
+
+=head2 _write_file( $fpath, $content, $mode )
+
+=cut
+
+sub _write_file {
+	my ($self, $fpath, $content, $mode) = @_;
+
+	# there's no need to open the output file in binary :utf-8 mode,
+	# as the YAML Dump() function returns UTF-8 encoded data (so it seems)
+
+	open(FILE, '>', $fpath)
+		|| croak "Can't open file $fpath for writing: $!";
+	flock(FILE, LOCK_EX);
+	print FILE $content;
+	close(FILE)
+		|| carp "Error closing file $fpath: $!";
+	chmod($mode, $fpath);
 }
 
 =head1 AUTHOR
